/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template is responsible to initialize the display hardware of the board
*   and to provide the necessary access to update the display content.
*   The color format of the framebuffer has to correspond to the color format
*   of the Graphics Engine.
*
*   Important: This file is intended to be used as a template. Please adapt the
*   implementation according your particular hardware.
*
*******************************************************************************/

#include "ewrte.h"
#include "ewgfx.h"
#include "ewextgfx.h"
#include "ewgfxdefs.h"

#include "ew_bsp_display.h"
#include "ew_bsp_clock.h"

#include "am_bsp.h"
#include "nema_core.h"
#include "nema_utils.h"
#include "am_hal_global.h"
#include "stdlib.h"
#include "string.h"
#include "am_util_stdio.h"

#include "nema_dc.h"
#include "nema_dc_mipi.h"
#include "am_devices_dsi_rm67162.h"
#include "am_devices_nemadc_rm67162.h"

#if ( EW_FRAME_BUFFER_COLOR_FORMAT  == EW_FRAME_BUFFER_COLOR_FORMAT_RGB565 )
  #define EW_BSP_NEMADC_PIXEL_FORMAT NEMADC_RGB565
  #define EW_BSP_NEMA_PIXEL_FORMAT   NEMA_RGB565
#else
  #error "selected EW_FRAME_BUFFER_COLOR_FORMAT not supported"
#endif

#if ( DISP_TYPE != RM69330_DSI )
  #error "Display type not supported"
#endif

#define EW_FRAME_BUFFER_SIZE  EW_FRAME_BUFFER_WIDTH * EW_FRAME_BUFFER_HEIGHT * EW_FRAME_BUFFER_DEPTH
#define EW_FRAME_BUFFER_ADDR  (unsigned int)FrameBuffer

static AM_SHARED_RW uint64_t FrameBuffer[ EW_FRAME_BUFFER_SIZE/8 ];

#if EW_USE_DOUBLE_BUFFER == 1
  #define EW_DOUBLE_BUFFER_ADDR  (unsigned int)DoubleBuffer

  static AM_SHARED_RW uint64_t DoubleBuffer[ EW_FRAME_BUFFER_SIZE/8 ];
#else
  #define EW_DOUBLE_BUFFER_ADDR  0
#endif

static nemadc_layer_t NemaDCLayer;

static volatile unsigned long PendingFramebuffer = 0;

/*******************************************************************************
* FUNCTION:
*   EwBspDisplayInit
*
* DESCRIPTION:
*   The function EwBspDisplayInit initializes the display hardware and returns
*   the display parameter.
*
* ARGUMENTS:
*   aGuiWidth,
*   aGuiHeight   - Size of the GUI in pixel.
*   aDisplayInfo - Display info data structure.
*
* RETURN VALUE:
*   Returns 1 if successful, 0 otherwise.
*
*******************************************************************************/
int EwBspDisplayInit( int aGuiWidth, int aGuiHeight, XDisplayInfo* aDisplayInfo )
{
  EW_UNUSED_ARG( aGuiWidth );
  EW_UNUSED_ARG( aGuiHeight );

  /* check and clean display info structure */
  if ( !aDisplayInfo )
    return 0;
  memset( aDisplayInfo, 0, sizeof( XDisplayInfo ));

  /* initialize Nema|dc */
  if ( nemadc_init() != 0)
  {
    EwPrint( "EwBspDisplayInit: nemadc_init failed.\n" );
    return 0;
  }

  if (( g_sDispCfg[g_eDispType].eInterface == IF_DSI ) || ( g_sDispCfg[g_eDispType].bUseDPHYPLL == true ))
  {
    uint8_t ui8LanesNum = g_sDsiCfg.ui8NumLanes;
    uint8_t ui8DbiWidth = g_sDsiCfg.eDbiWidth;
    uint32_t ui32FreqTrim = g_sDsiCfg.eDsiFreq;

    if ( am_hal_dsi_para_config( ui8LanesNum, ui8DbiWidth, ui32FreqTrim ) != 0 )
    {
      EwPrint( "EwBspDisplayInit: am_hal_dsi_para_config failed.\n" );
      return 0;
    }

    uint16_t ui16PanelResX = g_sDispCfg[g_eDispType].ui32PanelResX; //!< panel's max resolution
    uint16_t ui16PanelResY = g_sDispCfg[g_eDispType].ui32PanelResY; //!< panel's max resolution

    if (( ui16PanelResX != EW_FRAME_BUFFER_WIDTH ) || ( ui16PanelResY != EW_FRAME_BUFFER_HEIGHT ))
    {
      EwPrint( "EwBspDisplayInit: Incorrect framebuffer size.\n" );
      return 0;
    }

    /* initialize the display */
    am_devices_dsi_rm67162_init( MIPICFG_16RGB888_OPT0, EW_FRAME_BUFFER_WIDTH, EW_FRAME_BUFFER_HEIGHT, 0, 0 );
  }
  else
    return 0;

  NemaDCLayer.startx        = 0;
  NemaDCLayer.starty        = 0;
  NemaDCLayer.sizex         = EW_FRAME_BUFFER_WIDTH;
  NemaDCLayer.resx          = EW_FRAME_BUFFER_WIDTH;
  NemaDCLayer.sizey         = EW_FRAME_BUFFER_HEIGHT;
  NemaDCLayer.resy          = EW_FRAME_BUFFER_HEIGHT;
  NemaDCLayer.stride        = EW_FRAME_BUFFER_WIDTH * EW_FRAME_BUFFER_DEPTH;
  NemaDCLayer.format        = EW_BSP_NEMADC_PIXEL_FORMAT | NEMADC_MODULATE_A;
  NemaDCLayer.blendmode     = NEMADC_BL_SIMPLE;
  NemaDCLayer.alpha         = 0x80;
  NemaDCLayer.baseaddr_phys = EW_FRAME_BUFFER_ADDR;
  NemaDCLayer.baseaddr_virt = (void*)EW_FRAME_BUFFER_ADDR;

  nemadc_set_layer( 0, &NemaDCLayer );

  memset( (void*)EW_FRAME_BUFFER_ADDR, 0, EW_FRAME_BUFFER_SIZE );

  dsi_send_frame_single_start( NEMADC_OUTP_OFF );
  nemadc_wait_vsync();
  dsi_send_frame_single_end();

  /* return the current display configuration */
  aDisplayInfo->FrameBuffer    = (void*)EW_FRAME_BUFFER_ADDR;
  aDisplayInfo->DoubleBuffer   = (void*)EW_DOUBLE_BUFFER_ADDR;
  aDisplayInfo->BufferWidth    = EW_FRAME_BUFFER_WIDTH;
  aDisplayInfo->BufferHeight   = EW_FRAME_BUFFER_HEIGHT;
  aDisplayInfo->DisplayWidth   = EW_DISPLAY_WIDTH;
  aDisplayInfo->DisplayHeight  = EW_DISPLAY_HEIGHT;
  aDisplayInfo->UpdateMode     = EW_BSP_DISPLAY_UPDATE_NORMAL;

  return 1;
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayDone
*
* DESCRIPTION:
*   The function EwBspDisplayDone deinitializes the display hardware.
*
* ARGUMENTS:
*   aDisplayInfo - Display info data structure.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayDone( XDisplayInfo* aDisplayInfo )
{
  EW_UNUSED_ARG( aDisplayInfo );
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayGetUpdateArea
*
* DESCRIPTION:
*   The function EwBspDisplayGetUpdateArea returns the next update area
*   depending on the selected display mode:
*   In case of a synchroneous single-buffer, the function has to return the
*   the rectangular areas that correspond to the horizontal stripes (fields)
*   of the framebuffer.
*   In case of a scratch-pad buffer, the function has to return the subareas
*   that fit into the provided update rectangle.
*   During each display update, this function is called until it returns 0.
*
* ARGUMENTS:
*   aUpdateRect - Rectangular area which should be updated (redrawn).
*
* RETURN VALUE:
*   Returns 1 if a further update area can be provided, 0 otherwise.
*
*******************************************************************************/
int EwBspDisplayGetUpdateArea( XRect* aUpdateRect )
{
  EW_UNUSED_ARG( aUpdateRect );
  return 0;
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayWaitForCompletion
*
* DESCRIPTION:
*   The function EwBspDisplayWaitForCompletion is called from the Graphics Engine
*   to ensure that all pending activities of the display system are completed, so
*   that the rendering of the next frame can start.
*   In case of a double-buffering system, the function has to wait until the
*   V-sync has occured and the pending buffer is used by the display controller.
*   In case of an external display controller, the function has to wait until
*   the transfer (update) of the graphics data has been completed and there are
*   no pending buffers.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayWaitForCompletion( void )
{
  #if EW_USE_DOUBLE_BUFFER == 0

    CPU_LOAD_SET_IDLE();

    /* in case of single buffer, the DSI update has to be completed - otherwise
       content of framefuffer will be overwritten */
    while ( PendingFramebuffer )
    {
      nemadc_wait_vsync();
      dsi_send_frame_single_end();
      PendingFramebuffer = 0;
    }

    CPU_LOAD_SET_ACTIVE();

  #endif
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayCommitBuffer
*
* DESCRIPTION:
*   The function EwBspDisplayCommitBuffer is called from the Graphics Engine
*   when the rendering of a certain buffer has been completed.
*   The type of buffer depends on the selected framebuffer concept.
*   If the display is running in a double-buffering mode, the function is called
*   after each buffer update in order to change the currently active framebuffer
*   address. Changing the framebuffer address should be synchronized with V-sync.
*   If the system is using an external graphics controller, this function is
*   responsible to start the transfer of the framebuffer content.
*
* ARGUMENTS:
*   aAddress - Address of the framebuffer to be shown on the display.
*   aX,
*   aY       - Origin of the area which has been updated by the Graphics Engine.
*   aWidth,
*   aHeight  - Size of the area which has been updated by the Graphics Engine.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayCommitBuffer( void* aAddress, int aX, int aY, int aWidth, int aHeight )
{
  EW_UNUSED_ARG( aX );
  EW_UNUSED_ARG( aY );
  EW_UNUSED_ARG( aWidth );
  EW_UNUSED_ARG( aHeight );

  #if EW_USE_DOUBLE_BUFFER == 1

    CPU_LOAD_SET_IDLE();

    /* wait until pending framebuffer is transferred via DSI */
    while ( PendingFramebuffer )
    {
      nemadc_wait_vsync();
      dsi_send_frame_single_end();
      PendingFramebuffer = 0;
    }

    /* set new pending framebuffer address to be used on next V-sync */
    PendingFramebuffer = (uint32_t)aAddress;

    /* set new famebuffer address to display controller */
    NemaDCLayer.baseaddr_phys = PendingFramebuffer;
    NemaDCLayer.baseaddr_virt = (void*)PendingFramebuffer;
    nemadc_set_layer( 0, &NemaDCLayer );

    /* start the DSI transfer */
    dsi_send_frame_single_start( NEMADC_OUTP_OFF );

    CPU_LOAD_SET_ACTIVE();

  #else

    CPU_LOAD_SET_IDLE();

    /* just set pending framebuffer address to indicate new content */
    PendingFramebuffer = (uint32_t)aAddress;
    dsi_send_frame_single_start( NEMADC_OUTP_OFF );

    CPU_LOAD_SET_ACTIVE();

  #endif
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplaySetClut
*
* DESCRIPTION:
*   The function EwBspDisplaySetClut is called from the Graphics Engine
*   in order to update the hardware CLUT of the current framebuffer.
*   The function is only called when the color format of the framebuffer is
*   Index8 or LumA44.
*
* ARGUMENTS:
*   aClut - Pointer to a color lookup table with 256 entries.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplaySetClut( unsigned long* aClut )
{
  EW_UNUSED_ARG( aClut );
}


/* msy */
